use crate::data::error::ResultBoxError;
use crate::data::user::{User, UserID, UserPageStatus, AccountImageVisibility};
use crate::helpers::database;
use crate::constants::database_tables_names::USERS_TABLE;
use crate::data::user::UserPageStatus::PUBLIC;
use crate::helpers::friends_helper::are_friend;

/// User helper
///
/// @author Pierre Hubert

/// Get & return information about a user based on its ID
pub fn find_user_by_id(id: UserID) -> ResultBoxError<User> {
    exec_get_user_query(
        database::QueryInfo::new(USERS_TABLE).cond_i64("ID", id))
}

/// Get & return information about a user based on his email
pub fn find_user_by_email(email: &str) -> ResultBoxError<User> {
    exec_get_user_query(
        database::QueryInfo::new(USERS_TABLE).cond("mail", email))
}

/// Execute query & return result
fn exec_get_user_query(query: database::QueryInfo) -> ResultBoxError<User> {
    database::query_row(query, |res| {

        // Page status
        let page_status = if res.get_int64("pageouverte")? == 1 {
            UserPageStatus::OPEN
        } else if res.get_int64("public")? == 1 {
            UserPageStatus::PUBLIC
        } else {
            UserPageStatus::PRIVATE
        };

        // Account image visibility
        let account_image_visibility = match res.get_str("account_image_visibility")?.as_ref() {
            "friends" => AccountImageVisibility::FRIENDS,
            "comunic_users" => AccountImageVisibility::COMUNIC_USERS,
            "everyone" => AccountImageVisibility::EVERYONE,
            _ => unreachable!()
        };

        Ok(User {
            id: res.get_int64("ID")?,
            email: res.get_str("mail")?,
            password: res.get_str("password")?,
            first_name: res.get_str("prenom")?,
            last_name: res.get_str("nom")?,
            status: page_status,
            virtual_directory: res.get_optional_str("sous_repertoire")?,
            account_image_path: res.get_optional_str("account_image_path")?,
            account_image_visibility,
            public_friends_list: res.get_legacy_bool("liste_amis_publique")?,
            personal_website: res.get_optional_str("site_web")?,
            public_note: res.get_optional_str("public_note")?,
            block_comments_on_his_page: res.get_legacy_bool("bloquecommentaire")?,
            allow_posts_from_friends: res.get_legacy_bool("autoriser_post_amis")?,
            account_creation_time: res.get_date_as_time("date_creation")?
        })
    })
}

/// Check out whether a given id maps to a user or not
pub fn exists(id: UserID) -> ResultBoxError<bool> {
    Ok(database::QueryInfo::new(USERS_TABLE)
        .cond_i64("ID", id)
        .exec_count()? > 0)
}

/// Check if a given user can see another user's page
pub fn can_see_user_page(user_id: UserID, target_user: UserID) -> ResultBoxError<bool> {
    if user_id == target_user {
        return Ok(true);
    }

    let visibility = find_user_by_id(target_user)?.status;

    // Open page = OK
    if visibility ==  UserPageStatus::OPEN {
        return Ok(true);
    }

    // The user need to be signed in
    if user_id <= 0 {
        return Ok(false);
    }

    // Public Page = OK for signed in users
    if visibility == PUBLIC {
        return Ok(true);
    }

    // Check if the users are friends
    if !are_friend(user_id, target_user)? {
        return Ok(false);
    }

    return Ok(true);
}